using System;
using System.ComponentModel;
using System.Collections;

namespace InfiniTec.SystemUtilities.DataAccess
{
	/// <summary>
	/// Summary description for Element.
	/// </summary>
	public class Element: IElement
	{

		private PropertyCollection _Properties;
		protected string _Uri;

		public event PropertyChangeEventHandler PropertyChange;
		public event PropertyChangeEventHandler PropertyAdd;
		public event PropertyChangeEventHandler PropertyRemove;
		public event EventHandler PropertiesRemoved;
		public event ElementRenamedEventHandler Renamed;
		public event EventHandler Deleted;

		protected virtual void OnPropertyChange(PropertyChangeEventArgs e) 
		{
			if (PropertyChange != null) PropertyChange(this, e);
		}
		protected virtual void OnPropertyAdd(PropertyChangeEventArgs e) 
		{
			if ((PropertyAdd != null) && (e.Property.Changed)) PropertyAdd(this, e);
		}
		protected virtual void OnPropertyRemove(PropertyChangeEventArgs e) 
		{
			if (PropertyRemove != null) PropertyRemove(this, e);
		}
		protected virtual void OnPropertiesRemoved() 
		{
			if (PropertiesRemoved != null) PropertiesRemoved(this, new EventArgs());
		}

		public virtual PropertyCollection Properties 
		{
			get 
			{
				return _Properties;
			}
		}

		public string Uri 
		{
			get 
			{
				return _Uri; 
			}
		}

		private Element() {}

		public Element(string uri) 
		{
			_Properties = new PropertyCollection();
			_Properties.BeforeAdd += new PropertyChangeEventHandler(Properties_BeforeAdd);
			_Properties.BeforeClear += new CancelEventHandler(Properties_BeforeClear);
			_Properties.BeforeRemove += new PropertyChangeEventHandler(Properties_BeforeRemove);
			_Uri = uri.Trim(new Char[] {' ', '/', '#'}).ToLower();
		}

		protected void OnRenamed(ElementRenamedEventArgs e) 
		{
			if (Renamed != null) Renamed(this, e);
		}

		protected void OnDeleted() 
		{
			if (Deleted != null) Deleted(this, new EventArgs());
		}

		protected void Property_ValueChange(object sender, PropertyChangeEventArgs e)
		{
			OnPropertyChange(e);		
		}

		private void Properties_BeforeAdd(object source, PropertyChangeEventArgs e)
		{
			((Property)	e.Value).ValueChange += new PropertyChangeEventHandler(Property_ValueChange);
			OnPropertyAdd(e);
		}

		private void Properties_BeforeRemove(object source, PropertyChangeEventArgs e)
		{
			try 
			{
				((Property)	e.Value).ValueChange -= new PropertyChangeEventHandler(Property_ValueChange);
			}
			catch {}
			OnPropertyRemove(e);
		}

		private void Properties_BeforeClear(object source, CancelEventArgs e)
		{
			foreach (IProperty property in Properties) 
			{
				property.ValueChange -= new PropertyChangeEventHandler(Property_ValueChange);
			}
			OnPropertiesRemoved();
		}
	}
}
	