using System;

namespace InfiniTec.SystemUtilities.DataAccess
{
	/// <summary>
	/// Summary description for Property.
	/// </summary>
	public class Property: IProperty
	{
		private string _Name;
		private string _DataType;
		private object _Value;
		private bool _Changed;

		public event PropertyChangeEventHandler ValueChange;

		public virtual bool ReadOnly 
		{
			get 
			{
				return false;
			}
		}

		public void ResetChangedFlag() 
		{
			_Changed = false;
		}

		public Property(string name, string dataType, object value) 
		{
			_Name = name;
			_DataType = dataType;
			_Value = value;
			ResetChangedFlag();
		}
		public Property(string name, string dataType, object value, bool isDirty): this(name, dataType, value)
		{
			_Changed = isDirty;
		}

		protected virtual void OnValueChange(PropertyChangeEventArgs e) 
		{
			if (ValueChange != null) ValueChange(this, e);
		}

		public bool Changed 
		{
			get 
			{
				return _Changed;
			}
		}

		public string Name 
		{
			get { return _Name; }
		}

		public string DataType 
		{
			get { return _DataType; }
		}

		public object Value 
		{
			get { return _Value; }
			set {
				object oldValue;
				PropertyChangeEventArgs e;

				if (_Value != value) 
				{
					oldValue = _Value;

					if ((value is string) && (!(_Value is string))) 
					{
						_Value = ((IConvertible) value).ToType(_Value.GetType(), null);
						// _Value = _Value.GetType().GetMethod("Parse").Invoke(null, new object[] {value});
					}
					else _Value = value;
							
					e = new PropertyChangeEventArgs(this, value);
					try 
					{
						_Changed = true;
						OnValueChange(e);
						if (!e.Cancel)  
						{
							_Value = oldValue;
							_Changed = false;
						}
					}
					catch 
					{
						_Value = oldValue;
						_Changed = false;
						throw;
					}
				}
			}
		}

		public void SetProperty(string dataType, object value) 
		{
			_DataType = dataType;
			_Value = value;
		}

		public Property()
		{
		}
	}
}
