using System;
using System.ComponentModel;
using System.Collections;

namespace InfiniTec.SystemUtilities.DataAccess
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	public class PropertyCollection: IEnumerable
	{
		public event PropertyChangeEventHandler BeforeAdd;
		public event PropertyChangedEventHandler AfterAdd;
		public event CancelEventHandler BeforeClear;
		public event EventHandler AfterClear;
		public event PropertyChangeEventHandler BeforeRemove;
		public event PropertyChangedEventHandler AfterRemove;

		private bool _SuppressEvents;

		private Hashtable _Properties;


		public bool SuppressEvents 
		{
			get { return _SuppressEvents; }
			set { _SuppressEvents = value; }
		}

		public bool Contains(string name) 
		{
			return _Properties.Contains(name);
		}

		public IProperty  this[string name]
		{
			get
			{
				return (IProperty) _Properties[name];
			}
			set
			{
				_Properties[name] = value;
			}
		}

		protected virtual void OnBeforeAdd(PropertyChangeEventArgs e) 
		{
			if ((!SuppressEvents) && (BeforeAdd != null)) BeforeAdd(this, e);
		}

		protected virtual void OnAfterAdd(PropertyChangedEventArgs e) 
		{
			if ((!SuppressEvents) && (AfterAdd != null)) AfterAdd(this, e);
		}

		protected virtual void OnBeforeRemove(PropertyChangeEventArgs e) 
		{
			if ((!SuppressEvents) && (BeforeRemove != null)) BeforeRemove(this, e);
		}

		protected virtual void OnAfterRemove(PropertyChangedEventArgs e) 
		{
			if ((!SuppressEvents) && (AfterRemove != null)) AfterRemove(this, e);
		}

		protected virtual void OnBeforeClear(PropertyChangeEventArgs e) 
		{
			if ((!SuppressEvents) && (BeforeClear != null)) BeforeClear(this, e);
		}

		protected virtual void OnAfterClear() 
		{
			if ((!SuppressEvents) && (AfterClear != null)) AfterClear(this, new EventArgs());
		}

		public PropertyCollection(): base() 
		{
			_Properties = System.Collections.Specialized.CollectionsUtil.CreateCaseInsensitiveHashtable();
			_SuppressEvents = false;
		}

		public int Count 
		{
			get 
			{
				return _Properties.Count;
			}
		}

		public void Add(IProperty property) 
		{
			PropertyChangeEventArgs e;

			e = new PropertyChangeEventArgs(property, property.Value);
			OnBeforeAdd(e);
			if (!e.Cancel) 
			{
				_Properties.Add(property.Name, property);
				OnAfterAdd(new PropertyChangedEventArgs(property));
			}
		}

		public void Add(string name, object value)
		{
			Add(new Property(name, "", value));
		}

		public void Clear()
		{
			PropertyChangeEventArgs e;

			e = new PropertyChangeEventArgs(null, null);
			OnBeforeClear(e);
			if (!e.Cancel) 
			{
				_Properties.Clear();
				OnAfterClear();
			}
		}
		public void Remove(string name)
		{
			PropertyChangeEventArgs e;
			IProperty p;

			p = (IProperty) _Properties[name];
			e = new PropertyChangeEventArgs(p, null);

			OnBeforeRemove(e);
			if (!e.Cancel) 
			{
				_Properties.Remove(name);
				OnAfterRemove(new PropertyChangedEventArgs(p));
			}
		}
		#region IEnumerable Members

		public IEnumerator GetEnumerator()
		{
			return new PropertyCollectionEnumerator(_Properties);
		}

		#endregion
	}
}
