using System;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.ComponentModel;

namespace InfiniTec.SystemUtilities.DataAccess
{
	/// <summary>
	/// Summary description for PropertyPropertyDescriptor.
	/// </summary>
	public class PropertyPropertyDescriptor: PropertyDescriptor
	{
		private IProperty _property;
		private PropertyDescription _propertyDescription;
		private TranslationOptions _TranslationOption;

		public PropertyPropertyDescriptor(IProperty property): base(property.Name, new Attribute[] {})
		{
			_property = property;
			_TranslationOption = TranslationOptions.NamesOnly;
		}
		public PropertyPropertyDescriptor(IProperty property, PropertyDescription propertyDescription, TranslationOptions translationOption): this(property)
		{
			_property = property;
			_propertyDescription = propertyDescription;
			_TranslationOption = translationOption;
		}

		public override string Category
		{
			get
			{
				string ns;
				int index;
				ns = _property.Name;
				index = ns.LastIndexOfAny(new char[] {':', '/'});
				if (index > 0) return ns.Substring(0, index);
				else return "None";
			}
		}

		public override Type ComponentType
		{
			get
			{
				return _property.GetType();
			}
		}
		public override string DisplayName
		{
			get
			{
				string name;

				name = "";
				switch (_TranslationOption) 
				{
					case TranslationOptions.NamesAndTranslation: name = string.Format("{0} [{1}]", _property.Name, _propertyDescription.Name); break;
					case TranslationOptions.NamesOnly: name = _property.Name; break;
					case TranslationOptions.TranslationOnly: name = _propertyDescription.Name; break;
				}
				return name;
			}
		}

		public override string Name
		{
			get
			{
				return DisplayName;
			}
		}

		public override string Description
		{
			get
			{
				string description;

				if (_propertyDescription != null) 
				{
					description = string.Format("Original name: {0}\nFull name: {1}\nDatatype: {2}\nDescription: {3}", _property.Name, _propertyDescription.DisplayName, _property.DataType, _propertyDescription.Description);
				}
				else 
					description = string.Format("Datatype: {0}", _property.DataType);

				return description;
			}
		}


		public override object GetEditor(Type editorBaseType)
		{
			if (_property.DataType == "string") return new PropertyStringEditor();
			else return base.GetEditor (editorBaseType);
		}

		public override bool IsReadOnly
		{
			get
			{
				return _property.ReadOnly;
			}
		}
		public override Type PropertyType
		{
			get
			{
				return _property.Value.GetType();
			}
		}

		public override object GetValue(object component)
		{
			return ((IProperty) ((IElement) component).Properties[_property.Name]).Value;
		}
		public override void SetValue(object component, object value)
		{
			((IProperty) ((IElement) component).Properties[_property.Name]).Value = value;
		}


		public override bool CanResetValue(object component)
		{
			return false;
		}

		public override void ResetValue(object component)
		{

		}
		public override bool ShouldSerializeValue(object component)
		{
			return true;
		}


	}
}
