using System;
using System.Collections;
using System.Security.Cryptography.X509Certificates;
using System.Net;

namespace InfiniTec.WebDAV
{
	/// <summary>
	/// Summary description for CertificatePolicy.
	/// </summary>
	public class CertificatePolicy: ICertificatePolicy
	{
		public static ArrayList _ApprovedCertificates;
		public static event CertificateProblemOccuredEventHandler CertificateProblemOccured;

		protected virtual void OnCertificateProblemOccured(object source, CertificateProblemOccuredEventArgs e) 
		{
			if (CertificateProblemOccured != null) 
			{
				lock (CertificateProblemOccured) CertificateProblemOccured(source, e);
			}
		}

		static CertificatePolicy() 
		{
			_ApprovedCertificates = new ArrayList();
		}

		public CertificatePolicy()
		{
		}
		#region ICertificatePolicy Members

		public bool CheckValidationResult(ServicePoint srvPoint, X509Certificate certificate, System.Net.WebRequest request, int certificateProblem)
		{
			CertificateProblemOccuredEventArgs e;
			int i;

			// No error, accept the certificate
			if (certificateProblem == 0) return true;

			i = 0;

			while ((i < _ApprovedCertificates.Count) && (!certificate.Equals((X509Certificate) _ApprovedCertificates[i]))) i++;

			// Certificate was previously accepted, return true
			if (i < _ApprovedCertificates.Count) return true;

			e = new CertificateProblemOccuredEventArgs(certificate, request, certificateProblem);
			OnCertificateProblemOccured(this, e);

			// If the certificate was approved, add it to the list
			if (!e.Cancel) _ApprovedCertificates.Add(certificate);
			
			return (!e.Cancel);
		}

		#endregion
	}
}
