using System;
using System.Collections;
using System.Security.Cryptography;
using System.Text;
using InfiniTec.SystemUtilities.DataAccess;

namespace InfiniTec.WebDAV
{
	/// <summary>
	/// Summary description for Connection.
	/// </summary>
	public class Connection: IEnumerable
	{
		private Credential _Credential;
		private string _Server;
		private ConnectionProtocol _Protocol;
		private int _hashCode;
		protected ObjectFactory _objectFactory;
		private ArrayList ConnectedRoots;

		public Credential Credential 
		{
			get { return _Credential; }
			set { _Credential = value; UpdateHashCode(); }
		}

		public override int GetHashCode()
		{
			return _hashCode;
		}

		public ConnectionProtocol Protocol 
		{
			get 
			{
				return _Protocol;
			}
			set 
			{
				_Protocol = value;
			}
		}

		public string Server 
		{
			get {return _Server; }
			set {_Server = value; }
		}
	
		private void UpdateHashCode() 
		{
			SHA1Managed sha1 = new SHA1Managed();
			byte[] hash = sha1.ComputeHash(Encoding.UTF8.GetBytes(Credential.Username + Credential.Password + Credential.AuthenticationType.ToString()));
			_hashCode = Encoding.Default.GetString(hash).GetHashCode();
		}
		public Connection()
		{
			_objectFactory = new WebDAVObjectFactory(this);
			ConnectedRoots = new ArrayList();
		}

		public Connection(Credential credential): this()
		{
			Credential = credential;
		}

		public virtual IWebDAVFolder AddRoot(string root) 
		{
			StringBuilder sb;

			sb = new StringBuilder();
			sb.Append(_Protocol);
			sb.Append("://");
			sb.Append(_Server);
			sb.Append("/");
			sb.Append(root);

			return AddRootAbsolute(sb.ToString());
		}

		public virtual IWebDAVFolder AddRootAbsolute(string uri) 
		{
			IWebDAVFolder folder;

			folder = (IWebDAVFolder) _objectFactory.CreateFolder(uri);
			folder.Refresh();
			//folder.CompleteInitialization();

			ConnectedRoots.Add(folder);

			return folder;
		}

		#region IEnumerable Members

		public IEnumerator GetEnumerator()
		{
			// TODO:  Add Connection.GetEnumerator implementation
			return ConnectedRoots.GetEnumerator();
		}

		#endregion
	}
}
