using System;
using System.Xml;
using System.Collections;
using System.Collections.Specialized;
using InfiniTec.SystemUtilities.DataAccess;
using InfiniTec.WebDAV;

namespace InfiniTec.WebDAV
{
	/// <summary>
	/// Summary description for Utilities
	/// </summary>
	internal class Utilities
	{
		WebDAVLayer _webDAVLayer;
		IWebDAVElement _element;

		public Utilities(IWebDAVElement element, WebDAVLayer webDAVLayer) 
		{
			_webDAVLayer = webDAVLayer;
			_element = element;
		}

		public void HandlePropertyAdd(PropertyChangeEventArgs e, InitializationStatus initializationStatus) 
		{
			if (initializationStatus == InitializationStatus.Full)
			{
				try 
				{
				ForceFlush(true);
				} 
				catch 
				{
					e.Cancel = true;
					throw;
				}
			}
		}

		public void ForceFlush(bool onlyDirty) 
		{
			lock (_webDAVLayer) 
			{
				_webDAVLayer.PropPatch(_element, onlyDirty);
			}
		}

		public NameValueCollection Capabilities(string uri) 
		{
			return _webDAVLayer.Options(uri);
		}

		public void HandlePropertyChange(PropertyChangeEventArgs e, InitializationStatus initializationStatus) 
		{
			if (initializationStatus == InitializationStatus.Full)
			{
				lock (_webDAVLayer) 
				{
					try 
					{
						_webDAVLayer.PropPatch(_element, true);
					}
					catch 
					{
						e.Cancel = true;
						throw;
					}
				}
			}
		}

		public void Rename(string newName) 
		{
			_webDAVLayer.Rename(_element, newName);
		}


		public void Remove() 
		{
			lock (_webDAVLayer) 
			{
				_webDAVLayer.Delete(_element);
			}
		}
		public void UpdateProperties(IElement dest, IElement source) 
		{
			dest.Properties.SuppressEvents = true;

			try 
			{
				foreach (Property property in dest.Properties) 
				{
					if (!source.Properties.Contains(property.Name)) 
					{
						source.Properties.Remove(property.Name);
					}
				}

				foreach (IProperty property in source.Properties) 
				{
					if (dest.Properties.Contains(property.Name))
					{
						((Property) dest.Properties[property.Name]).SetProperty(property.DataType, property.Value);
					}
					else 
					{
						dest.Properties.Add(property);
					}
				}
			} 
			finally 
			{
				dest.Properties.SuppressEvents = false;
			}
		}
	}
}
