using System;
using System.Web;
using System.IO;
using System.Diagnostics;
using System.Text;
using System.Xml;
using System.Net;
using System.Collections;
using System.Collections.Specialized;

namespace InfiniTec.WebDAV
{
	/// <summary>
	/// Summary description for WebRequest.
	/// </summary>
	public class WebRequest
	{
		string _ConnectionGroupName;
		Response[] _Responses;
		static CookieCollection _Cookies;

		public Response[] Responses 
		{
			get { return _Responses; }
		}

		static WebRequest() 
		{
			ServicePointManager.CertificatePolicy = new CertificatePolicy();
		}

		public string ConnectionGroupName 
		{
			get { return _ConnectionGroupName; }
			set { _ConnectionGroupName = (value != null) ? value : ""; }
		}

		public WebRequest(string connectionGroupName)
		{
			ConnectionGroupName = connectionGroupName;
		}

		public XmlDocument Request(string uri, string method, NameValueCollection headers, System.Xml.XmlDocument request, NameValueCollection responseheaders, Credential credential) 
		{
			return DoRequest(uri, method, headers, request, responseheaders, credential);
		}

		public XmlDocument Request(string uri, string method, System.Xml.XmlDocument request, Credential credential) 
		{
			return DoRequest(uri, method, null, request, null, credential);
		}
		public XmlDocument Request(string uri, string method, NameValueCollection headers, Credential credential) 
		{
			return DoRequest(uri, method, headers, null, null, credential);
		}

		public XmlDocument Request(string uri, string method, Credential credential) 
		{
			return DoRequest(uri, method, null, null, null, credential);
		}
		public XmlDocument Request(string uri, string method, System.Xml.XmlDocument request, int depth, Credential credential) 
		{
			NameValueCollection headers;

			headers = new NameValueCollection();
			headers.Add("Depth", depth.ToString());
			return DoRequest(uri, method, headers, request, null, credential);
		}

		public XmlDocument Request(string uri, string method, NameValueCollection headers, System.Xml.XmlDocument request, Credential credential) 
		{
			return DoRequest(uri, method, headers, request, null, credential);
		}
		
		public XmlDocument Request(string uri, string method, NameValueCollection headers, Credential credential, NameValueCollection responseheaders) 
		{
			return DoRequest(uri, method, headers, null, responseheaders, credential);
		}

		private static void DoFormbasedAuthentication(string uri, Credential credential, string connectionGroup)
		{
			string server;
			HttpWebRequest request;
			HttpWebResponse response;
			byte[] body;
			Stream stream;
			string result;

			if (_Cookies == null) 
			{
				try 
				{
					server = uri.Substring(0, uri.IndexOf("/", 8)) + "/exchweb/bin/auth/owaauth.dll";

					request = GetRequestObject(server, "POST", connectionGroup, null);
					request.CookieContainer = new CookieContainer();
					request.ContentType = "application/x-www-form-urlencoded";

					body = Encoding.UTF8.GetBytes(string.Format("destination={0}&username={1}&password={2}", uri, credential.Username, credential.Password));
			
					request.ContentLength = body.Length;

					stream = request.GetRequestStream();
					stream.Write(body, 0, body.Length);
					stream.Close();

					response = (HttpWebResponse) request.GetResponse();

					if (response.Cookies.Count < 2) throw new AuthenticationException("Failed to login to OWA. Be sure your domain\\username and password are correct.");

					_Cookies = response.Cookies;
				} 
				catch (Exception ex) 
				{
					throw new AuthenticationException("Failed to login to OWA. The following error occured: " + ex.Message, ex);
				}
			}
		}

		internal static HttpWebRequest GetRequestObject(string uri, string method, string connectionGroup, Credential credential) 
		{
			HttpWebRequest webrequest;
			string[] credentials;
			CredentialCache cache;
			Uri _uri;


			//				System.Windows.Forms.MessageBox.Show(strXml);
									 
			// Initiate a new WebRequest to the given URI.

			_uri = new Uri(uri);
			webrequest = (HttpWebRequest) System.Net.WebRequest.Create(_uri);

			if (credential != null) 
			{
				if (credential.AuthenticationType == AuthenticationType.FormBasedAuthentication) DoFormbasedAuthentication(uri, credential, connectionGroup);
				else 
				{
					cache = new CredentialCache();
 
					if (credential.Username != "") 
					{
						credentials = credential.Username.Split(new char[] {'\\'}, 2);
						if (credentials.Length == 2) 
							cache.Add(_uri, credential.AuthenticationType.ToString(), new System.Net.NetworkCredential(credentials[1], credential.Password, credentials[0]));
						else 
							cache.Add(_uri, credential.AuthenticationType.ToString(), new System.Net.NetworkCredential(credentials[0], credential.Password, ""));
						webrequest.Credentials = cache;
					}
					else 
						webrequest.Credentials = CredentialCache.DefaultCredentials;
				}
			}

			if (_Cookies != null) 
			{
				webrequest.CookieContainer = new CookieContainer();
				webrequest.CookieContainer.Add(new Uri(uri), _Cookies);
			}
			webrequest.Method = method;
			webrequest.ConnectionGroupName = connectionGroup;
			webrequest.UnsafeAuthenticatedConnectionSharing = true;
			webrequest.PreAuthenticate = true;
			webrequest.KeepAlive = true;
			webrequest.Pipelined = true;
			webrequest.UserAgent = "Exchange Admin";
			webrequest.ProtocolVersion = HttpVersion.Version11;
			webrequest.ServicePoint.UseNagleAlgorithm = false;
			webrequest.ServicePoint.Expect100Continue = false;
			webrequest.ContentType = "text/xml";

			return webrequest;
		}

		internal XmlDocument DoRequest(string uri, string method, NameValueCollection headers, System.Xml.XmlDocument request, NameValueCollection responseheaders, Credential credential)
		{
			HttpWebRequest webrequest;
			System.Xml.XmlDocument doc;
			System.IO.Stream stream;
			System.Text.UTF8Encoding encoder;
			byte[] content;
			HttpWebResponse response;
			string strXml;
			string[] rangeData;

			try 
			{
				if (request != null) strXml = request.OuterXml;
				else strXml = String.Empty;
			
				webrequest = GetRequestObject(uri, method, this.ConnectionGroupName, credential);

				webrequest.SendChunked = true;
				//webrequest.Headers.Add("Translate", "f");

				// If headers should be send along, add them to the
				// collection.
				if (headers != null) 
				{
					foreach (string header in headers) 
					{
						if (header.ToLower() == "range") 
						{
							rangeData = headers[header].Split(new char[] {'=', '-'});
							webrequest.AddRange(rangeData[0], int.Parse(rangeData[1]), int.Parse(rangeData[2]));
						}
						else 
							webrequest.Headers.Add(header, headers[header]);
					}
				}
				// Set the length of the request.

				if (request != null) 
				{
					// Send the Request to the server.
					encoder = new System.Text.UTF8Encoding(false);
					content = encoder.GetBytes(strXml);
					webrequest.ContentLength = content.Length;

					stream = webrequest.GetRequestStream();
					stream.Write(content, 0, content.Length);
					stream.Close();
				}
				else 
					webrequest.ContentLength = 0;

				// Get the response stream.
				response = (HttpWebResponse) webrequest.GetResponse();

				if (responseheaders != null) 
				{
					responseheaders.Clear();
					responseheaders.Add(response.Headers);
				}
			
				doc = new System.Xml.XmlDocument();
				if (response.ContentLength != 0) 
				{
					doc.Load(response.GetResponseStream());
					if ((int) response.StatusCode == 207) ParseResponse(doc);
				}
				response.Close();
				return doc;
			} 
			catch (Exception ex) 
			{
				LogException(ex);
				throw;
			}
		}

		private void ParseResponse(XmlDocument doc) 
		{
			XmlNamespaceManager nsmgr;
			XmlNodeList nodes;

			nsmgr = new XmlNamespaceManager(doc.CreateNavigator().NameTable);
			nsmgr.AddNamespace("a", "DAV:");
			
			nodes = doc.SelectNodes("//a:response", nsmgr);

			_Responses = new Response[nodes.Count];

			for (int i = 0; i<nodes.Count; i++) 
			{
				_Responses[i] = new Response(nodes[i].SelectSingleNode("//a:status", nsmgr).InnerText,
					nodes[i].SelectSingleNode("a:href", nsmgr).InnerText);
			}
		}

		private void LogException(Exception ex) 
		{
			System.Diagnostics.Debug.WriteLine(ex.ToString());
		}
	}
}
